using System;
using System.Collections;
using System.Runtime.Serialization;
using System.Security.Permissions;
using System.Configuration;

namespace OnlinePhotoShop
{
	/// <summary>
	/// Class that stores a number of shopping items associated with a unique key. This class and the items stored in it are serializable and allow the class to be stored within a Session object even when using ASP Session State Service.
	/// </summary>
	/// <requirement>F:product_shopping_cart</requirement>
	/// <requirement>F:product_customize</requirement>
	/// <requirement>F:product_options</requirement>
	[Serializable()]
	public class ShoppingCart : Hashtable, ISerializable
	{
		/// <summary>
		/// Private enumerator for unique numbers. This enumerator is incremented every time a new item is added to the list.
		/// </summary>
		private int mNextKey;
		/// <summary>
		/// Private storage for unique identifier string (SessionID). This string is used to generate server side file names for uploaded images.
		/// </summary>
		private string mID;

		#region ISerializable Members

		/// <summary>
		/// The serialization function.
		/// </summary>
		/// <param name="info">The object to be populated with serialization information.</param>
		/// <param name="context">The destination context of the serialization.</param>
		[SecurityPermissionAttribute(SecurityAction.Demand,SerializationFormatter=true)]
		override public void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("NextKey", mNextKey);
			info.AddValue("ID", mID);
		}

		/// <summary>
		/// The deserialization constructor.
		/// </summary>
		/// <param name="info">The object populated with serialization information.</param>
		/// <param name="context">The source context of the serialization.</param>
		public ShoppingCart(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			mNextKey = (int) info.GetValue("NextKey", typeof(int));
			mID = (string) info.GetValue("ID", typeof(string));
		}

		#endregion

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="id">SessionID of current HTTP request.</param>
		public ShoppingCart(string id)
		{
			mNextKey = 0;
			mID = id;
		}

		/// <summary>
		/// Adds a new item to the shopping cart.
		/// </summary>
		/// <param name="product">Product identifier.</param>
		/// <param name="option">Selected product options.</param>
		/// <param name="image">Image file to customize the item with.</param>
		/// <param name="quantity">Quantity of the ordered item.</param>
		public void AddItem(string product, string option, System.Web.HttpPostedFile image, int quantity)
		{
			if (image != null)
			{
				// pick a new key
				int key = mNextKey++;

				// save the image to the upload directory
				string extension = image.FileName.Substring(image.FileName.LastIndexOf("."));
				string store = ConfigurationSettings.AppSettings["ImageStore"];
				string path = store + "\\" + mID + "_" + key.ToString() + extension;
				image.SaveAs(path);

				// add the item to the order list
				this[key] = new ShoppingItem(product, option, quantity, image.FileName, path);
			}
		}

		/// <summary>
		/// Computes the total of all items in the shopping cart.
		/// </summary>
		/// <param name="productCatalog">Product catalog file.</param>
		/// <returns>Total cost of all items in the shopping cart.</returns>
		public double GetTotalOrder(string productCatalog)
		{
			double total = 0;
			ProductParser parser = new ProductParser(productCatalog);

			IDictionaryEnumerator i = this.GetEnumerator();
			while(i.MoveNext())
			{
				ShoppingItem item = (ShoppingItem) i.Value;

				// get the product details
				System.Collections.Hashtable details = parser.OptionDetails(item.Product, item.Option);

				// add to total
				double price = System.Convert.ToDouble((string) details["price"]);
				total += (price * item.Quantity);
			}

			return total;
		}

		/// <summary>
		/// Computes the total weight of all items in the shopping cart.
		/// </summary>
		/// <param name="productCatalog">Product catalog file.</param>
		/// <returns>Total weight of all items in the shopping cart.</returns>
		public double GetTotalWeight(string productCatalog)
		{
			double total = 0;
			ProductParser parser = new ProductParser(productCatalog);

			IDictionaryEnumerator i = this.GetEnumerator();
			while(i.MoveNext())
			{
				ShoppingItem item = (ShoppingItem) i.Value;

				// get the product details
				System.Collections.Hashtable details = parser.OptionDetails(item.Product, item.Option);

				// add to total
				double weight = System.Convert.ToDouble((string) details["weight"]);
				total += (weight * item.Quantity);
			}

			return total;
		}

	}
}
