using System;
using NUnit.Framework;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Collections;
using System.Xml;
using System.Configuration;

namespace OnlinePhotoShop
{
	/// <summary>
	/// Unit Test for Online Photo Shop
	/// </summary>
	[TestFixture]
	public class UnitTest
	{
		public UnitTest()
		{
			// 
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// Creates a temporary XML product catalog with a given set of products and options.
		/// </summary>
		/// <param name="products">Number of product entries in the catalog.</param>
		/// <param name="options">Number of options for each product.</param>
		/// <returns>File path of the generated XML product catalog.</returns>
		string CreateProductCatalog(int products, int options)
		{
			// Get path for a temporary file
			string file = System.IO.Path.GetTempFileName();

			// create a temp XML file
			XmlTextWriter writer=new XmlTextWriter(File.CreateText(file));
			// Bug in .NET Framework 1.1, ReadInnerXML works only indented
			writer.Formatting = System.Xml.Formatting.Indented;

			writer.WriteStartDocument();
			writer.WriteStartElement("doc");
			writer.WriteStartElement("products");

			for (int p = 1; p <= products; p++)
			{
				writer.WriteStartElement("product");
				writer.WriteAttributeString(
					"id", "product_id" + p.ToString());
				writer.WriteAttributeString(
					"name", "product_name" + p.ToString());

				writer.WriteElementString(
					"summary", "product_summary" + p.ToString());
				writer.WriteElementString(
					"image", "product_image" + p.ToString());

				for (int o = 1; o <= options; o++)
				{
					writer.WriteStartElement("option");
					writer.WriteAttributeString(
						"id", "option_id" + p.ToString() + o.ToString());
					writer.WriteAttributeString(
						"name", "option_name" + p.ToString() + o.ToString());
					writer.WriteAttributeString(
						"price", p.ToString() + "." + o.ToString());
					writer.WriteElementString("summary",
						"option_summary" + p.ToString() + o.ToString());
					writer.WriteElementString("weight",
						"0." + p.ToString() + o.ToString());
					writer.WriteEndElement();  // option
				}

				writer.WriteEndElement();  // product
			}

			writer.WriteEndElement();  // products
			writer.WriteEndElement();  // doc
			writer.WriteEndDocument();
			writer.Close();

			return file;
		}

		/// <summary>
		/// Test the ProductList and OptionList method of ProductParser.
		/// </summary>
		/// <requirement>F:order_products</requirement>
		/// <requirement>F:products_browse</requirement> 
		/// <requirement>F:product_options</requirement> 
		[Test]
		public void ProductParserListsTest()
		{
			string tmpFile = this.CreateProductCatalog(10, 5);

			ProductParser parser = new ProductParser(tmpFile);

			// check product list
			SortedList p = parser.ProductList();
			Assertion.AssertEquals(p.Count, 10);
			for (int i = 0; i < p.Count; i++)
			{
				Assertion.AssertEquals(
					p["product_id" + (i + 1).ToString()],
					"product_name" + (i + 1).ToString());
			}

			// check one option list, pick one from the center
			SortedList o = parser.OptionList("product_id3");
			Assertion.AssertEquals(o.Count, 5);
			for (int i = 0; i < o.Count; i++)
			{
				Assertion.AssertEquals(
					o["option_id3" + (i + 1).ToString()], 
					"option_name3" + (i + 1).ToString() + " - "
						+ (3 + (i + 1) / 10.0f).ToString("C"));
			}

			File.Delete(tmpFile);
		}

		/// <summary>
		/// Test the ProductDetails and OptionDetails method of ProductParser.
		/// </summary>
		/// <requirement>F:order_products</requirement>
		/// <requirement>F:products_browse</requirement> 
		/// <requirement>F:product_options</requirement> 
		[Test]
		public void ProductParserDetailsTest()
		{
			string tmpFile = this.CreateProductCatalog(20, 3);

			ProductParser parser = new ProductParser(tmpFile);

			// check product details of the 20th entry
			Hashtable p = parser.ProductDetails("product_id20");
			Assertion.AssertEquals((string) p["name"], "product_name20");
			Assertion.AssertEquals(
				(string) p["summary"], "product_summary20");
			Assertion.AssertEquals(
				(string) p["image"], "product_image20");

			// check some option details (3rd product, 2nd option)
			Hashtable o = parser.OptionDetails(
				"product_id3", "option_id32");
			Assertion.AssertEquals((string) o["name"], "option_name32");
			Assertion.AssertEquals(
				(string) o["summary"], "option_summary32");
			Assertion.AssertEquals(
				(string) o["price"], (3.2f).ToString());

			File.Delete(tmpFile);
		}

		/// <summary>
		/// Helper function that serializes an object into a stream and returns the
		/// deserialized object from the same stream.
		/// </summary>
		/// <param name="obj">Object for serialization.</param>
		/// <returns>Deserialized object.</returns>
		/// <requirement>C:online_shop_stateservice</requirement>
		Object SerializationTest(Object obj)
		{
			// Opens a file and serializes the object into it
			// in binary format
			string tmpFile = System.IO.Path.GetTempFileName();
			Stream stream = File.Open(tmpFile, FileMode.Create);
			BinaryFormatter bformatter = new BinaryFormatter();
			bformatter.Serialize(stream, obj);
			stream.Close();

			//Opens file and deserializes the object from it
			stream = File.Open(tmpFile, FileMode.Open);
			bformatter = new BinaryFormatter();
			Object deserialized = bformatter.Deserialize(stream);
			stream.Close();

			return deserialized;
		}

		/// <summary>
		/// Tests the ShoppingItem class.
		/// </summary>
		/// <requirement>F:product_shopping_cart</requirement> 
		[Test]
		public void ShoppingItemTest()
		{
			// test constructor
			ShoppingItem item = new ShoppingItem("product", "option", 123, "clientPath", "serverPath");
			Assertion.AssertEquals(item.Product, "product");
			Assertion.AssertEquals(item.Option, "option");
			Assertion.AssertEquals(item.Quantity, 123);
			Assertion.AssertEquals(item.ClientPath, "clientPath");
			Assertion.AssertEquals(item.ServerPath, "serverPath");

			// test (de)serialization
			ShoppingItem item2 = (ShoppingItem) this.SerializationTest(item);
			Assertion.AssertEquals(item2.Product, "product");
			Assertion.AssertEquals(item2.Option, "option");
			Assertion.AssertEquals(item2.Quantity, 123);
			Assertion.AssertEquals(item2.ClientPath, "clientPath");
			Assertion.AssertEquals(item2.ServerPath, "serverPath");
		}

		/// <summary>
		/// Tests the ShoppingCart class.
		/// </summary>
		/// <requirement>F:product_shopping_cart</requirement>
		/// <requirement>F:product_customize</requirement>
		[Test]
		public void ShoppingCartTest()
		{
			// test constructor
			ShoppingItem item = new ShoppingItem("product", "option", 123, "clientPath", "serverPath");

			ShoppingCart cart = new ShoppingCart("id");

			// AddItem requires an HttpPostedFile
			// if null nothing should happen
			cart.AddItem("product", "option", null, 123);
			Assertion.AssertEquals(cart.Count, 0);

			// Add two keys to test serialization
			cart.Add("key1", new ShoppingItem("product1", "option1", 1, "clientPath1", "serverPath1"));
			cart.Add("key2", new ShoppingItem("product2", "option2", 2, "clientPath2", "serverPath2"));

			ShoppingCart cart2 = (ShoppingCart) this.SerializationTest(cart);
			Assertion.AssertEquals(cart2.Count, 2);
			// Check one property (already tested in ShoppingItemTest)
			Assertion.AssertEquals(((ShoppingItem)cart2["key1"]).Product, "product1");
			Assertion.AssertEquals(((ShoppingItem)cart2["key2"]).Product, "product2");
		}

		/// <summary>
		/// Tests the ShoppingItemView class.
		/// </summary>
		/// <requirement>F:product_shopping_cart</requirement>
		[Test]
		public void ShoppingItemViewTest()
		{
			string tmpFile = this.CreateProductCatalog(2, 2);

			// create a shopping item to format for viewing
			ShoppingItem item = new ShoppingItem("product_id1", "option_id12", 10, "clientPath", "serverPath");
  
			// create the formatted view class
			ShoppingItemView view = new ShoppingItemView(tmpFile, 555, item);
    
			Assertion.AssertEquals(view.Key, "555");
			Assertion.AssertEquals(view.Product, "product_name1");
			Assertion.AssertEquals(view.Option, "option_name12");
			Assertion.AssertEquals(view.Quantity, 10);
			Assertion.AssertEquals(view.Subtotal, 12);  // 1.20 * 10
			Assertion.AssertEquals(view.Price, 12.ToString("C"));
			Assertion.AssertEquals(view.Image, "clientPath");

			File.Delete(tmpFile);
		}

	}
}
